# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import shark.utils.fields
import shark.utils.id_generators


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Customer',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('number', shark.utils.id_generators.IdField(unique=True, max_length=32, blank=True)),
                ('address', shark.utils.fields.OldAddressField(verbose_name='address')),
                ('language', shark.utils.fields.LanguageField(blank=True, max_length=5, choices=[(b'af', b'Afrikaans'), (b'ar', b'Arabic'), (b'ast', b'Asturian'), (b'az', b'Azerbaijani'), (b'bg', b'Bulgarian'), (b'be', b'Belarusian'), (b'bn', b'Bengali'), (b'br', b'Breton'), (b'bs', b'Bosnian'), (b'ca', b'Catalan'), (b'cs', b'Czech'), (b'cy', b'Welsh'), (b'da', b'Danish'), (b'de', b'German'), (b'el', b'Greek'), (b'en', b'English'), (b'en-au', b'Australian English'), (b'en-gb', b'British English'), (b'eo', b'Esperanto'), (b'es', b'Spanish'), (b'es-ar', b'Argentinian Spanish'), (b'es-mx', b'Mexican Spanish'), (b'es-ni', b'Nicaraguan Spanish'), (b'es-ve', b'Venezuelan Spanish'), (b'et', b'Estonian'), (b'eu', b'Basque'), (b'fa', b'Persian'), (b'fi', b'Finnish'), (b'fr', b'French'), (b'fy', b'Frisian'), (b'ga', b'Irish'), (b'gl', b'Galician'), (b'he', b'Hebrew'), (b'hi', b'Hindi'), (b'hr', b'Croatian'), (b'hu', b'Hungarian'), (b'ia', b'Interlingua'), (b'id', b'Indonesian'), (b'io', b'Ido'), (b'is', b'Icelandic'), (b'it', b'Italian'), (b'ja', b'Japanese'), (b'ka', b'Georgian'), (b'kk', b'Kazakh'), (b'km', b'Khmer'), (b'kn', b'Kannada'), (b'ko', b'Korean'), (b'lb', b'Luxembourgish'), (b'lt', b'Lithuanian'), (b'lv', b'Latvian'), (b'mk', b'Macedonian'), (b'ml', b'Malayalam'), (b'mn', b'Mongolian'), (b'mr', b'Marathi'), (b'my', b'Burmese'), (b'nb', b'Norwegian Bokmal'), (b'ne', b'Nepali'), (b'nl', b'Dutch'), (b'nn', b'Norwegian Nynorsk'), (b'os', b'Ossetic'), (b'pa', b'Punjabi'), (b'pl', b'Polish'), (b'pt', b'Portuguese'), (b'pt-br', b'Brazilian Portuguese'), (b'ro', b'Romanian'), (b'ru', b'Russian'), (b'sk', b'Slovak'), (b'sl', b'Slovenian'), (b'sq', b'Albanian'), (b'sr', b'Serbian'), (b'sr-latn', b'Serbian Latin'), (b'sv', b'Swedish'), (b'sw', b'Swahili'), (b'ta', b'Tamil'), (b'te', b'Telugu'), (b'th', b'Thai'), (b'tr', b'Turkish'), (b'tt', b'Tatar'), (b'udm', b'Udmurt'), (b'uk', b'Ukrainian'), (b'ur', b'Urdu'), (b'vi', b'Vietnamese'), (b'zh-cn', b'Simplified Chinese'), (b'zh-hans', b'Simplified Chinese'), (b'zh-hant', b'Traditional Chinese'), (b'zh-tw', b'Traditional Chinese')])),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='updated')),
            ],
            options={
                'abstract': False,
                'verbose_name': 'customer',
                'verbose_name_plural': 'customers',
            },
            bases=(models.Model,),
        ),
    ]



from decimal import Decimal

from django.conf import settings
from django.template.loader import render_to_string


class DirectDebit(object):

    def __init__(self, **kwargs):
        def kwargs_get(name):
            '''Get SEPA creditor information from kwargs and fallback
            to SEPA_<ARGUMENT_NAME>.'''
            try:
                return kwargs[name]
            except KeyError:
                return getattr(settings, 'SEPA_%s' % name.upper())
        self.id = kwargs['id']
        self.creditor_id = kwargs_get('creditor_id')
        self.creditor_name = kwargs_get('creditor_name')
        self.creditor_country = kwargs_get('creditor_country')
        self.creditor_iban = kwargs_get('creditor_iban')
        self.creditor_bic = kwargs_get('creditor_bic')
        self.due_date = kwargs_get('due_date')
        self.mandate_type = kwargs.get('mandate_type', 'CORE')
        # B2B = without revocation for preauthorized payments
        assert self.mandate_type in ('CORE', 'COR1', 'B2B')
        self.sequence_type = kwargs.get('sequence_type', 'FRST')
        assert self.sequence_type in ('FRST', 'RCUR')
        self.transactions = list(kwargs.get('transactions', []))
        self.batch_booking = kwargs.get('batch_booking', False)

    def add_transaction(self, transaction):
        self.transactions.append(transaction)

    @property
    def control_sum(self):
        return sum(txn.amount for txn in self.transactions)

    def render_xml(self):
        '''
        Create SEPA XML document according to ISO20222.
        '''
        return render_to_string('sepa/direct_debit.xml', {
            'dd': self
        })


class Transaction(object):

    def __init__(self, debitor_name, debitor_country, debitor_bic, debitor_iban,
            reference, amount, mandate_id, mandate_date):
        self.debitor_name = debitor_name
        self.debitor_country = debitor_country
        self.debitor_iban = debitor_iban
        self.debitor_bic = debitor_bic
        self.reference = reference
        self.amount = Decimal(amount)
        self.mandate_id = mandate_id
        self.mandate_date = mandate_date



#!/usr/bin/env python

import os
from setuptools import Command
from setuptools import find_packages, setup

def read(*p):
    '''Utility function to read files relative to the project root'''
    return open(os.path.join(os.path.dirname(__file__), *p)).read()

def get_version():
    '''Get __version__ information from __init__.py without importing it'''
    import re
    VERSION_RE = r'^__version__\s*=\s*[\'"]([^\'"]+)[\'"]'
    VERSION_PATTERN = re.compile(VERSION_RE, re.MULTILINE)
    m = VERSION_PATTERN.search(read('shark', '__init__.py'))
    if m:
        return m.group(1)
    else:
        raise RuntimeError('Could not get __version__ from shark/__init__.py')

# Prevent "TypeError: 'NoneType' object is not callable" when running tests.
# (http://www.eby-sarna.com/pipermail/peak/2010-May/003357.html)
try:
    import multiprocessing
except ImportError:
    pass

class DjangoTestCommand(Command):
    description = "run unit test using Django management command"
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        # Prepare DJANGO_SETTINGS_MODULE and PYTHONPATH
        import os, sys
        os.environ['DJANGO_SETTINGS_MODULE'] = 'test_settings'
        sys.path[0:0] = [os.path.dirname(os.path.dirname(__file__))]
        # Execute management command 'test'
        from django.core.management import execute_from_command_line
        execute_from_command_line(['manage.py', 'test'])

setup(
    name='shark',
    version=get_version(),
    description='Django based billing application',
    long_description=read('README.rst'),
    author='Michael P. Jung',
    author_email='michael.jung@terreon.de',
    license='BSD',
    keywords='shark',
    url='http://github.com/bikeshedder/shark',
    packages=find_packages(),
    tests_require=['Django'],
    install_requires=['Django', 'dinbrief', 'pypdf2', 'django-taggit', 'django-localflavor', 'django-countries', 'python-magic', 'wand', 'django-composite-field'],
    setup_requires=['wheel'],
    cmdclass={
        'test': DjangoTestCommand,
    },
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
)



# Generated by Django 2.2.2 on 2019-06-05 12:39

from django.db import migrations, models
import django.db.models.deletion
import shark.utils.fields


class Migration(migrations.Migration):

    dependencies = [
        ('vendor', '0001_initial'),
        ('customer', '0004_add_various_fields'),
        ('documents', '0003_thumbnails'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='document',
            name='original',
        ),
        migrations.AddField(
            model_name='document',
            name='customer',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='customer.Customer', verbose_name='customer'),
        ),
        migrations.AddField(
            model_name='document',
            name='direction',
            field=models.CharField(choices=[('internal', 'internal'), ('inbound', 'inbound'), ('outbound', 'outbound')], default='inbound', max_length=10, verbose_name='direction'),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='document',
            name='original_filename',
            field=models.TextField(blank=True, editable=False, null=True),
        ),
        migrations.AddField(
            model_name='document',
            name='recipient',
            field=shark.utils.fields.OldAddressField(blank=True),
        ),
        migrations.AddField(
            model_name='document',
            name='sender',
            field=shark.utils.fields.OldAddressField(blank=True),
        ),
        migrations.AddField(
            model_name='document',
            name='source',
            field=models.CharField(blank=True, choices=[('email', 'email'), ('download', 'download'), ('mail', 'mail'), ('fax', 'fax'), ('receipt', 'receipt'), ('self', 'self')], help_text='Where does this document come from?', max_length=10, verbose_name='original'),
        ),
        migrations.AddField(
            model_name='document',
            name='type',
            field=models.CharField(choices=[('invoice', 'invoice'), ('payment_reminder', 'payment reminder'), ('bank_statement', 'bank statement'), ('quote', 'quote'), ('misc', 'miscellaneous'), ('sepa_dd_mandate', 'SEPA direct debit mandate')], default='invoice', max_length=20, verbose_name='direction'),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='document',
            name='vendor',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='vendor.Vendor', verbose_name='vendor'),
        ),
    ]



from django.core.mail import EmailMessage
from django.template.loader import render_to_string


def send_templated_mail(subject_template, body_template, dictionary, *args, **kwargs):
    subject = render_to_string(subject_template, dictionary).strip()
    body = render_to_string(body_template, dictionary)
    message = EmailMessage(subject, body, *args, **kwargs)
    message.send()



# Generated by Django 2.1.1 on 2018-09-10 17:52

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('documents', '0002_meta_changes'),
    ]

    operations = [
        migrations.AddField(
            model_name='document',
            name='thumbnail_large',
            field=models.ImageField(default='', editable=False, upload_to='documents'),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='document',
            name='thumbnail_medium',
            field=models.ImageField(default='', editable=False, upload_to='documents'),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='document',
            name='thumbnail_small',
            field=models.ImageField(default='', editable=False, upload_to='documents'),
            preserve_default=False,
        ),
    ]



from autocomplete_light import shortcuts as autocomplete_light

from shark.customer import models


class CustomerAutocomplete(autocomplete_light.AutocompleteModelBase):
    search_fields = ['number', 'address']
    model = models.Customer

    def choices_for_request(self):
        if not self.request.user.is_superuser:
            self.choices = self.choices.none()
        return super(CustomerAutocomplete, self).choices_for_request()

    def choice_label(self, instance):
        return u'%s - %s' % (instance.number, ', '.join(instance.address.split('\n')))


autocomplete_light.register(CustomerAutocomplete)



import unicodecsv as csv

from django import forms
from django.forms.utils import ErrorList
from django.utils.html import mark_safe
from django.utils.text import format_lazy
from django.utils.translation import ugettext
from django.utils.translation import ugettext_lazy

from shark.billing.models import InvoiceItem
from shark.customer.models import Customer


class ItemForm(forms.ModelForm):
    customer_number = forms.CharField(max_length=Customer._meta.get_field('number').max_length)

    class Meta:
        model = InvoiceItem
        fields = ['customer_number', 'quantity', 'sku', 'text',
                'begin', 'end', 'price', 'unit', 'discount', 'vat_rate']

    def resolve_customer(self, customer_dict):
        # mark item form errnous if customer does not exist
        customer_number = self.cleaned_data['customer_number']
        customer = customer_dict[customer_number]
        if not customer:
            self.errors.setdefault('customer_number', ErrorList())
            self.errors['customer_number'].append(
                    ugettext('Customer does not exist.'))
        self.cleaned_data['customer'] = customer

    def save(self, commit=True):
        item = super(ItemForm, self).save(commit=False)
        item.customer = self.cleaned_data['customer']
        if commit:
            item.save()
        return item


class ImportItemsForm(forms.Form):
    file = forms.FileField(
            help_text=ugettext_lazy(
                'Please specify a CSV file to be imported. '
                'The following fields are supported:'))
    delimiter = forms.ChoiceField(
            choices=(
                (',', ', (LibreOffice)'),
                (';', '; (Excel)')
            ))
    encoding = forms.ChoiceField(
            choices=(
                ('utf-8', 'utf-8 (LibreOffice)'),
                ('latin1', 'latin1 (Excel)')
            ))

    def __init__(self, *args, **kwargs):
        super(ImportItemsForm, self).__init__(*args, **kwargs)
        file_field = self.fields['file']
        file_field.help_text = format_lazy('{} {}',
                file_field.help_text,
                ', '.join(ItemForm._meta.fields))

    def clean(self):
        super(ImportItemsForm, self).clean()
        if 'file' not in self.cleaned_data or \
                'delimiter' not in self.cleaned_data or \
                'encoding' not in self.cleaned_data:
            return
        item_forms = []
        errnous_rows = []
        class Dialect(csv.excel):
            delimiter = str(self.cleaned_data['delimiter'])
        reader = csv.DictReader(self.cleaned_data['file'],
                dialect=Dialect, encoding=self.cleaned_data['encoding'])
        customer_dict = {}
        # create item forms
        for row, data in enumerate(reader, 2):
            for field_name in ItemForm._meta.fields:
                field_value = data.get(field_name, '')
                if field_value == '':
                    try:
                        field = ItemForm.base_fields[field_name]
                    except KeyError:
                        continue
                    data[field_name] = field.initial
            item_form = ItemForm(data)
            item_forms.append((row, item_form))
            # get customer numbers from all rows
            if item_form.is_valid() and 'customer_number' in item_form.cleaned_data:
                customer_number = item_form.cleaned_data['customer_number']
                customer_dict[customer_number] = None
        # load all customers
        for customer in Customer.objects.filter(number__in=customer_dict.keys()):
            customer_dict[customer.number] = customer
        # prepare list of errnous rows
        for row, item_form in item_forms:
            if item_form.is_valid() and 'customer_number' in item_form.cleaned_data:
                # resolve customer
                item_form.resolve_customer(customer_dict)
            if not item_form.is_valid():
                errnous_rows.append((row, item_form.errors))
        # prepare error message for rows
        if errnous_rows:
            self.errnous_rows = errnous_rows
            errors = self.errors.setdefault('file', ErrorList())
            for row, errnous_fields in errnous_rows:
                row_errors = []
                for field_name, field_errors in errnous_fields.items():
                    row_errors.append('<li>%s: %s</li>' % (
                        field_name, ' '.join(field_errors)))
                errors.append(mark_safe('%s %d:<ul>%s</ul>' % (
                    ugettext('Line'), row, ''.join(row_errors))))
        else:
            self.cleaned_data['items'] = [
                    item_form.save(commit=False)
                    for row, item_form in item_forms]
        return self.cleaned_data



# Generated by Django 2.2.2 on 2019-06-05 12:39

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('banking', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='transaction',
            name='currency',
            field=models.CharField(default='EUR', max_length=3),
        ),
    ]



from xml.sax.saxutils import escape

from django.conf import settings
from django.contrib import admin
from django.urls import reverse
from django.utils.html import format_html, format_html_join, mark_safe
from django.utils.translation import ugettext_lazy as _
from django.utils.translation import ungettext
from django.utils.translation import override as trans_override

from shark.sepa import models
from shark.utils.mail import send_templated_mail


class DirectDebitMandateAdmin(admin.ModelAdmin):
    list_display = ['customer', 'address_html', 'iban', 'bic']
    list_filter = ['created', 'signed']
    search_fields = ['number', 'address', 'created']
    raw_id_fields = ['document']
    autocomplete_fields = ('customer',)

    def address_html(self, instance):
        return format_html_join(mark_safe('<br>'), '{}', ((line,) for line in instance.address_lines))
    address_html.short_description = _('address')


class DirectDebitTransactionAdmin(admin.ModelAdmin):
    list_display = ['customer', 'mandate', 'amount', 'invoice', 'batch', 'created']
    search_fields = ['customer__name']
    #raw_id_fields = ['customer', 'mandate', 'invoice', 'accounting_transaction', 'batch']
    raw_id_fields = ['customer', 'mandate', 'invoice', 'batch']


class DirectDebitBatchAdmin(admin.ModelAdmin):
    list_display = ['uuid', 'created', 'executed', 'sepaxml_link']
    list_filter = ['created', 'executed']
    actions = ['send_pre_notifications']

    def sepaxml_link(self, instance):
        return format_html('<a href="{}">{}</a>',
                reverse('sepa_admin:directdebitbatch_sepaxml', args=(instance.pk,)),
                'Download')
    sepaxml_link.short_description = 'SEPA XML'

    def send_pre_notifications(self, request, queryset):
        for batch in queryset:
            transaction_list = list(batch.directdebittransaction_set.all() \
                    .select_related('mandate'))
            for transaction in transaction_list:
                with trans_override(transaction.customer.language):
                    send_templated_mail(
                        subject_template='sepa/pre_notification_email_subject.txt',
                        body_template='sepa/pre_notification_email_body.txt',
                        dictionary={
                            'batch': batch,
                            'transaction': transaction,
                            'customer': transaction.customer,
                            'mandate': transaction.mandate,
                            'creditor_id': settings.SHARK['SEPA']['CREDITOR_ID'],
                        },
                        from_email=settings.SHARK['SEPA']['PRE_NOTIFICATION_EMAIL_FROM'],
                        to=[address.email for address in transaction.customer.email_set.all()],
                        bcc=settings.SHARK['SEPA']['PRE_NOTIFICATION_EMAIL_BCC'],
                    )
            self.message_user(request, ungettext(
                'Send %(count)d pre-notification for batch %(batch_uuid)s',
                'Send %(count)d pre-notifications for batch %(batch_uuid)s',
                len(transaction_list)
            ) % {
                'count': len(transaction_list),
                'batch_uuid': batch.uuid,
            })
    send_pre_notifications.short_description = _('Send pre-notifications for selected batches')


admin.site.register(models.DirectDebitMandate, DirectDebitMandateAdmin)
admin.site.register(models.DirectDebitTransaction, DirectDebitTransactionAdmin)
admin.site.register(models.DirectDebitBatch, DirectDebitBatchAdmin)



# Generated by Django 2.1.1 on 2018-09-10 13:18

from django.db import migrations, models
import django_hashedfilenamestorage.storage
import shark.utils.date


class Migration(migrations.Migration):

    dependencies = [
        ('documents', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='document',
            name='date',
            field=models.DateField(default=shark.utils.date.today, help_text='Date as written on the document.', verbose_name='date'),
        ),
        migrations.AlterField(
            model_name='document',
            name='file',
            field=models.FileField(storage=django_hashedfilenamestorage.storage.HashedFilenameFileSystemStorage(), upload_to='documents', verbose_name='file'),
        ),
        migrations.AlterField(
            model_name='document',
            name='original',
            field=models.CharField(blank=True, choices=[('email', 'email'), ('download', 'download'), ('mail', 'mail'), ('fax', 'fax'), ('receipt', 'receipt')], help_text='Where does this document come from?', max_length=10, verbose_name='original'),
        ),
        migrations.AlterField(
            model_name='document',
            name='received',
            field=models.DateField(default=shark.utils.date.today, help_text='Date when the document was received.', verbose_name='received'),
        ),
    ]



from functools import partial
import uuid

from django.conf import settings
from django.db import models
from django.template.loader import render_to_string
from django.utils.translation import ugettext_lazy as _
from django_countries.fields import CountryField
from localflavor.generic.models import BICField
from localflavor.generic.models import IBANField

from shark.utils.settings import get_settings_value

from . import sepaxml
from .utils import anonymize_iban


class DirectDebitMandate(models.Model):
    customer = models.ForeignKey('customer.Customer',
            on_delete=models.CASCADE,
            verbose_name=_('customer'),
            related_name='direct_debit_mandate_set')
    reference = models.CharField(_('mandate reference'),
            max_length=35, unique=True, blank=True, null=True)
    name = models.CharField(max_length=100)
    street = models.CharField(max_length=100)
    postal_code = models.CharField(max_length=20)
    city = models.CharField(max_length=100)
    country = CountryField(default='DE')
    iban = IBANField('IBAN',
            help_text='International Bank Account Number')
    bic = BICField('BIC',
            help_text='Bank Identifier Code')
    bank_name = models.CharField(max_length=50, blank=True)
    created = models.DateTimeField(_('created'), auto_now_add=True)
    updated = models.DateTimeField(_('updated'), auto_now=True)
    signed = models.DateField(blank=True, null=True)
    revoked = models.DateField(_('revoked'), blank=True, null=True)
    last_used = models.DateField(_('last_used'), blank=True, null=True)
    TYPE_CORE = 'CORE'
    TYPE_COR1 = 'COR1'
    TYPE_B2B = 'B2B'
    TYPE_CHOICES = (
        (TYPE_CORE, 'CORE'),
        (TYPE_COR1, 'COR1'),
        (TYPE_B2B, 'B2B'),
    )
    type = models.CharField(max_length=4, choices=TYPE_CHOICES)
    document = models.ForeignKey('documents.Document',
            on_delete=models.CASCADE,
            verbose_name=_('signed document'),
            blank=True, null=True)

    class Meta:
        verbose_name = _('SEPA direct debit mandate')
        verbose_name_plural = _('SEPA direct debit mandates')

    def __str__(self):
        if self.reference:
            return self.reference
        else:
            return '<no reference>'

    @property
    def address_lines(self):
        return [
            self.name,
            self.street,
            self.postal_code + ' ' + self.city,
            self.get_country_display()
        ]

    @property
    def anonymized_iban(self):
        return anonymize_iban(self.iban)


class DirectDebitTransaction(models.Model):
    customer = models.ForeignKey('customer.Customer',
            on_delete=models.CASCADE,
            verbose_name=_('customer'),
            related_name='direct_debit_transaction_set')
    mandate = models.ForeignKey('sepa.DirectDebitMandate', verbose_name=_('SEPA DD mandate'),
            on_delete=models.CASCADE)
    reference = models.CharField(max_length=140)
    amount = models.DecimalField(_('amount'),
            max_digits=11, decimal_places=2)
    invoice = models.ForeignKey('billing.Invoice', verbose_name=_('invoice'),
            on_delete=models.CASCADE,
            blank=True, null=True)
    batch = models.ForeignKey('sepa.DirectDebitBatch', verbose_name=_('SEPA DD batch'),
            on_delete=models.CASCADE)
    created = models.DateTimeField(_('created'), auto_now_add=True)

    @classmethod
    def from_invoice(cls, invoice):
        mandate = DirectDebitMandate.objects \
                .filter(customer=invoice.customer, revoked=None) \
                .order_by('-created')[:1].get()
        obj = cls(
                customer=invoice.customer,
                mandate=mandate,
                reference=invoice.number,
                amount=invoice.gross,
                invoice=invoice)
        return obj


def get_default_creditor_id():
    return get_settings_value('SEPA.CREDITOR_ID', '')

def get_default_creditor_name():
    return get_settings_value('SEPA.CREDITOR_NAME', '')

def get_default_creditor_country():
    return get_settings_value('SEPA.CREDITOR_COUNTRY', '')

def get_default_creditor_iban():
    return get_settings_value('SEPA.CREDITOR_IBAN', '')

def get_default_creditor_bic():
    return get_settings_value('SEPA.CREDITOR_BIC', '')


class DirectDebitBatch(models.Model):
    '''
    This model is used to process multiple SEPA DD transactions
    together. This is typically achieved by generating a SEPA XML
    file.
    '''
    uuid = models.UUIDField(_('UUID'), default=uuid.uuid4)
    creditor_id = models.CharField(_('creditor id'), max_length=20,
            default=get_default_creditor_id)
    creditor_name = models.CharField(_('creditor name'), max_length=70,
            default=get_default_creditor_name)
    creditor_country = models.CharField(_('creditor country'), max_length=2,
            default=get_default_creditor_country)
    creditor_iban = IBANField(_('creditor IBAN'),
            default=get_default_creditor_iban)
    creditor_bic = BICField(_('creditor BIC'),
            default=get_default_creditor_bic)
    due_date = models.DateField(_('due date'),
            help_text=_('Must be min. 5 TARGET dates in the future for the first transaction and 2 target days in the future for recurring transactions.'))
    mandate_type = models.CharField(_('mandate type'),
            max_length=4, choices=DirectDebitMandate.TYPE_CHOICES)
    SEQUENCE_TYPE_FRST = "FRST"
    SEQUENCE_TYPE_RCUR = "RCUR"
    SEQUENCE_TYPE_CHOICES = (
        (SEQUENCE_TYPE_FRST, 'FRST'),
        (SEQUENCE_TYPE_RCUR, 'RCUR'),
    )
    sequence_type = models.CharField(_('sequence type'),
            max_length=4, choices=SEQUENCE_TYPE_CHOICES)
    created = models.DateTimeField(_('created'),
            auto_now_add=True)
    executed = models.DateTimeField(_('executed'),
            blank=True, null=True)

    @property
    def transactions(self):
        return list(self.directdebittransaction_set.all())

    def render_sepa_xml(self):
        '''
        Create SEPA XML document according to ISO20222.
        '''
        dd = sepaxml.DirectDebit(
            id=self.uuid.hex,
            creditor_id=self.creditor_id,
            creditor_name=self.creditor_name,
            creditor_country=self.creditor_country,
            creditor_iban=self.creditor_iban,
            creditor_bic=self.creditor_bic,
            due_date=self.due_date,
            mandate_type=self.mandate_type,
            sequence_type=self.sequence_type,
            transactions=[
                sepaxml.Transaction(
                    debitor_name=txn.mandate.name,
                    debitor_country=txn.mandate.country,
                    debitor_iban=txn.mandate.iban,
                    debitor_bic=txn.mandate.bic,
                    reference=settings.SHARK['SEPA']['TRANSACTION_REFERENCE_PREFIX'] + txn.reference,
                    amount=txn.amount,
                    mandate_id=txn.mandate.reference,
                    mandate_date=txn.mandate.signed
                )
                for txn in self.directdebittransaction_set.all()
            ]
        )
        return dd.render_xml()
